(************************************************************************
 * Compclx.PAS V6.0  This source is for Kylix/D6+ CLX use ONLY          *
 *                                                                      *
 * Legal stuff:                                                         *
 *  Copyright(c) 1995-2001, Baskin-Kerr Trust under license to SPIS Ltd *
 *  Email: software@spis.co.nz  Web: www.spis.co.nz                     *
 *                                                                      *
 * This source is largely derived from portions of the QDBCtrls.pas     *
 * unit distributed by Borland as part of CLX, and which carries the    *
 * following notice:                                                    *
 *                                                                      *
 * Kylix and Delphi Cross-Platform Visual Component Library             *
 *                                                                      *
 * Copyright (c) 2000, 2001 Borland Software Corporation                *
 *                                                                      *
 * This file may be distributed and/or modified under the terms of the  *
 * GNU General Public License (GPL) version 2 as published by the Free  *
 * Software Foundation and appearing at                                 *
 * http://www.borland.com/kylix/gpl.html.                               *
 *                                                                      *
 * Licensees holding a valid Borland No-Nonsense License for this       *
 * Software may use this file in accordance with such license, which    *
 * appears in the file license.txt that came with this Software.        *
 ************************************************************************
 Release history:
   June 2001: V6.0, for TCompress 6.01. Apart from the global rename of
     the TDBMemo and TDBImage controls to TCDBMemo and TCDBImage, you
     will find all changes or additions flagged by {} in the relevant lines
*)

unit CompCLX;

{$R-,H+,X+}

interface
uses SysUtils, QStdCtrls, Compress, QDBCtrls, DB, Classes, QControls, QGraphics,
  Qt, QForms, Types;

type

{ TCDBMemo }

  TCDBMemo = class(TCustomMemo)
  private
{}    FCompressSource: TCompress;
{}    FCompressionMethod: TCompressionMethod;
    FDataLink: TFieldDataLink;
    FAutoDisplay: Boolean;
    FFocused: Boolean;
    FMemoLoaded: Boolean;
    FTextLocked: Boolean;
    FDatasetLocked: Boolean;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetAutoDisplay(Value: Boolean);
    procedure SetFocused(Value: Boolean);
    procedure SetReadOnly(const Value: Boolean); reintroduce; 
    procedure UpdateData(Sender: TObject);
  protected
    procedure Change; override;
    procedure DblClick; override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    procedure LoadMemo; virtual;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property Field: TField read GetField;
  published
{}    property CompressSource: TCompress Read FCompressSource Write FCompressSource;
{}    property CompressionMethod: TCompressionMethod Read FCompressionmethod Write FCompressionMethod;
    property Align;
    property Alignment;
    property Anchors;
    property AutoDisplay: Boolean read FAutoDisplay write SetAutoDisplay default True;
    property BorderStyle;
    property Color;
    property Constraints;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ScrollBars;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property WantTabs;
    property WordWrap;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyString;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

{ TCDBImage }

  TCDBImage = class(TCustomControl)
  private
{}    FCompressSource: TCompress;
{}    FCompressionMethod: TCompressionMethod;
    FDataLink: TFieldDataLink;
    FPicture: TPicture;
    FBorderStyle: TControlBorderStyle;
    FAutoDisplay: Boolean;
    FStretch: Boolean;
    FCenter: Boolean;
    FPictureLoaded: Boolean;
    procedure DataChange(Sender: TObject);
    function GetBorderSize: Integer;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure PictureChanged(Sender: TObject);
    procedure SetAutoDisplay(Value: Boolean);
    procedure SetBorderStyle(Value: TControlBorderStyle);
    procedure SetCenter(Value: Boolean);
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetPicture(Value: TPicture);
    procedure SetReadOnly(Value: Boolean);
    procedure SetStretch(Value: Boolean);
    procedure UpdateData(Sender: TObject);
  protected
    procedure BoundsChanged; override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure DblClick; override;
    function GetClientRect: TRect; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Paint; override;
    procedure TextChanged; override;
    property BorderSize: Integer read GetBorderSize;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    procedure LoadPicture;
    procedure PasteFromClipboard;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property Field: TField read GetField;
    property Picture: TPicture read FPicture write SetPicture;
  published
{}    property CompressSource: TCompress Read FCompressSource Write FCompressSource;
{}    property CompressionMethod: TCompressionMethod Read FCompressionmethod Write FCompressionMethod;
    property Align;
    property Anchors;
    property AutoDisplay: Boolean read FAutoDisplay write SetAutoDisplay default True;
    property BorderStyle: TControlBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property Center: Boolean read FCenter write SetCenter default True;
    property Color default clBase;
    property Constraints;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor default False;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property Stretch: Boolean read FStretch write SetStretch default False;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyString;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

{} // TCompress -- the next four methods are required for TCompress interfacing:
procedure WriteCompressedString(CompressSource: TCompress; CompressionMethod: TCompressionMethod ;
            const str: string;Field: TBlobField);
var outS: TStream;
    inS: TMemoryStream;
begin
   outS := Field.DataSet.CreateBlobStream(Field,bmWrite);
   inS := TMemoryStream.create;
   try
     inS.write(str[1],Length(str));
     inS.seek(0,soFromBeginning);
     CompressSource.Compress(outS,inS,CompressionMethod)
   finally
     outS.free;
     inS.free;
   end;
end;

{} // TCompress:
function ReadCompressedString(CompressSource:TCompress;Field:TBlobField): string;
var outS: TMemoryStream;
    inS: TStream;
begin
   outS := TMemoryStream.create;
   inS := Field.DataSet.CreateBlobStream(Field,bmRead);
   try
     CompressSource.Expand(outS,inS);
     with outS do
     begin
       setLength(result,outS.size);
       seek(0,soFromBeginning);
       read(result[1],outS.size);
     end;
   finally
     outS.free;
     inS.free;
   end;
end;

{}{ Paradox graphic BLOB header  -- excerpted from DB.PAS -- beware of changes! }
type
  TGraphicHeader = record
    Count: Word;                { Fixed at 1 }
    HType: Word;                { Fixed at $0100 }
    Size: Longint;              { Size not including header }
  end;

{}// TCompress:
procedure WriteCompressedPicture(CompressSource: TCompress;CompressionMethod: TCompressionMethod;
            Picture: TPicture; Field: TBlobField);
var
  Header: TGraphicHeader;
  outS: TStream;
  inS: TMemoryStream;
begin
  outS := Field.DataSet.CreateBlobStream(Field,bmWrite);
  inS := TMemoryStream.create;
  try
    if (Field.DataType = ftGraphic) or (Field.Datatype = ftTypedBinary) then
    begin
      Header.Count := 1;
      Header.HType := $0100;
      Header.Size := 0;
      inS.Write(Header, SizeOf(Header));
      Picture.Bitmap.SaveToStream(inS);
      Header.Size := inS.position - SizeOf(Header);
      inS.position := 0;
      inS.Write(Header, SizeOf(Header));
    end else
      Picture.Bitmap.SaveToStream(inS);
    inS.Position := 0;
    CompressSource.Compress(outS,inS,CompressionMethod)
   finally
     outS.free;
     inS.free;
   end;
end;

{}// TCompress:
procedure ReadCompressedPicture(CompressSource: TCompress; Picture: TPicture; Field: TBlobField);
var
  Size: Longint;
  Header: TGraphicHeader;
  outS: TMemoryStream;
  inS: TStream;
begin
   outS := TMemoryStream.create;
   inS := Field.DataSet.CreateBlobStream(Field,bmRead);
   try
     CompressSource.Expand(outS,inS);
     Size := OutS.Size;
     if Size >= SizeOf(TGraphicHeader) then
     begin
       OutS.Position := 0;
       OutS.Read(Header, SizeOf(Header));
       if (Header.Count <> 1) or (Header.HType <> $0100) or
         (Header.Size <> Size - SizeOf(Header)) then
         OutS.Position := 0;
     end;
     Picture.Bitmap.LoadFromStream(OutS);
   finally
     outS.free;
     inS.free;
   end;
end;


{ TCDBMemo }

constructor TCDBMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FAutoDisplay := True;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
{}  if (FDataLink.Field <> nil) and FDataLink.Field.IsBlob then
{}    TBlobField(FDataLink.Field).Transliterate := False;
  inherited ReadOnly := True;
end;

destructor TCDBMemo.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TCDBMemo.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TCDBMemo.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TCDBMemo.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if FMemoLoaded then
  begin
    if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
      not FDataLink.Field.IsValidChar(Key) then
    begin
      Beep;
      Key := #0;
    end;
    if Key = #27 then
      FDataLink.Reset;
  end else
  begin
    if Key = #13 then LoadMemo;
    Key := #0;
  end;
end;

procedure TCDBMemo.Change;
begin
  if not FTextLocked then
  begin
    FDatasetLocked := True;
    try
      if FDataLink.CanModify then
      begin
        FDataLink.Edit;
        FDataLink.Modified;
        FMemoLoaded := True;
      end;
    finally
      FDatasetLocked := False;
    end;
  end;
  inherited Change;
end;

procedure TCDBMemo.DblClick;
begin
  if not FMemoLoaded then
    LoadMemo
  else
    inherited;
end;

procedure TCDBMemo.DoEnter;
begin
  SetFocused(True);
  inherited;
  if SysLocale.FarEast and FDataLink.CanModify then
    inherited ReadOnly := False;
end;

procedure TCDBMemo.DoExit;
begin
  try
    FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;
  SetFocused(False);
  inherited;
end;

function TCDBMemo.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TCDBMemo.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TCDBMemo.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TCDBMemo.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TCDBMemo.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TCDBMemo.SetReadOnly(const Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TCDBMemo.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TCDBMemo.LoadMemo;
var
  cp: TCaretPos;
begin
  if not FMemoLoaded and Assigned(FDataLink.Field) and FDataLink.Field.IsBlob then
  begin
    try
{}    if CompressSource=nil then
        Lines.Text := FDataLink.Field.AsString
{}    else
{}      Lines.Text := ReadCompressedString(CompressSource,TBlobField(FDataLink.Field));
      FMemoLoaded := True;
    except
      { Memo too large }
      on E:EInvalidOperation do
        Lines.Text := Format('(%s)', [E.Message]);
    end;
    cp.Col := 0;
    cp.Line := 0;
    CaretPos := cp;
    EditingChange(Self);
  end;
end;

procedure TCDBMemo.DataChange(Sender: TObject);
begin
  if not FDatasetLocked then
  begin
    FTextLocked := True;
    try
      if FDataLink.Field <> nil then
        if FDataLink.Field.IsBlob then
        begin
          if FAutoDisplay or (FDataLink.Editing and FMemoLoaded) then
          begin
            FMemoLoaded := False;
            LoadMemo;
          end else
          begin
            Text := Format('(%s)', [FDataLink.Field.DisplayLabel]);
            FMemoLoaded := False;
          end;
        end else
        begin
          if FFocused and FDataLink.CanModify then
            Text := FDataLink.Field.Text
          else
            Text := FDataLink.Field.DisplayText;
          FMemoLoaded := True;
        end
      else
      begin
        if csDesigning in ComponentState then Text := Name else Text := '';
        FMemoLoaded := False;
      end;
      inherited ReadOnly := not (FDataLink.CanModify and FMemoLoaded);
    finally
      FTextLocked := False;
    end;
    Repaint;
  end;
end;

procedure TCDBMemo.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not (FDataLink.CanModify and FMemoLoaded);
end;

procedure TCDBMemo.UpdateData(Sender: TObject);
begin
{}  if CompressSource=nil then
      FDataLink.Field.AsString := Text
{}  else
{}    WriteCompressedString(CompressSource, CompressionMethod, Text,TBlobField(FDataLink.Field));
end;

procedure TCDBMemo.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
  begin
    FFocused := Value;
    if not Assigned(FDataLink.Field) or not FDataLink.Field.IsBlob then
      FDataLink.Reset;
  end;
end;

procedure TCDBMemo.SetAutoDisplay(Value: Boolean);
begin
  if FAutoDisplay <> Value then
  begin
    FAutoDisplay := Value;
    if Value then LoadMemo;
  end;
end;

function TCDBMemo.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TCDBMemo.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{ TCDBImage }

constructor TCDBImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque, csReplicatable, csFramed];
  Width := 105;
  Height := 105;
  TabStop := True;
  Color := clBase;
  FPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FBorderStyle := bsSingle;
  FAutoDisplay := True;
  FCenter := True;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TCDBImage.Destroy;
begin
  FPicture.Free;
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

function TCDBImage.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TCDBImage.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TCDBImage.GetBorderSize: Integer;
begin
  if FBorderStyle = bsSingle then
    Result := 1
  else
    Result := 0;
end;

function TCDBImage.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TCDBImage.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TCDBImage.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TCDBImage.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TCDBImage.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TCDBImage.SetAutoDisplay(Value: Boolean);
begin
  if FAutoDisplay <> Value then
  begin
    FAutoDisplay := Value;
    if Value then LoadPicture;
  end;
end;

procedure TCDBImage.SetBorderStyle(Value: TControlBorderStyle);
begin
  if FBorderStyle <> Value then
  begin
    FBorderStyle := Value;
    Invalidate;
  end;
end;

procedure TCDBImage.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    Invalidate;
  end;
end;

procedure TCDBImage.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TCDBImage.SetStretch(Value: Boolean);
begin
  if FStretch <> Value then
  begin
    FStretch := Value;
    Invalidate;
  end;
end;

procedure TCDBImage.Paint;
var
  Size: TSize;
  R: TRect;
  S: string;
  XOfs: Integer;
  YOfs: Integer;
  DrawPict: TPicture;
  Form: TCustomForm;

  procedure DrawFrame;
  var
    ARect: TRect;
  begin
    if FBorderStyle = bsSingle then
    begin
      ARect := Rect(0, 0, Width, Height);
      DrawShadePanel(Canvas, ARect, True, 1);
      InflateRect(ARect, -1, -1);
      Canvas.Pen.Color := clBlack;
      Canvas.Pen.Width := 1;
      Canvas.Pen.Style := psSolid;
      Canvas.Brush.Style := bsClear;
      Canvas.Rectangle(ARect);
      InflateRect(ARect, -1, -1);
      QPainter_setClipRect(Canvas.Handle, @ARect);
    end;
  end;

begin
  DrawFrame;
  with Canvas do
  begin
    Brush.Color := Self.Color;
    Brush.Style := bsSolid;
    FillRect(ClientRect);
    if FPictureLoaded or (csPaintCopy in ControlState) then
    begin
      DrawPict := TPicture.Create;
      try
        if (csPaintCopy in ControlState) and
          Assigned(FDataLink.Field) and FDataLink.Field.IsBlob then
          DrawPict.Assign(FDataLink.Field)
        else
          DrawPict.Assign(Picture);
        if Stretch then
          if (DrawPict.Graphic = nil) or DrawPict.Graphic.Empty then
            FillRect(ClientRect)
          else
            StretchDraw(ClientRect, DrawPict.Graphic)
        else
        begin
          R := Rect(0, 0, DrawPict.Width, DrawPict.Height);
          if Center then
          begin
            XOfs := (ClientWidth - DrawPict.Width) div 2 + BorderSize;
            YOfs := (ClientHeight - DrawPict.Height) div 2 + BorderSize;
            OffsetRect(R, XOfs, YOfs);
            if XOfs > 0 then begin
              FillRect(Rect(BorderSize, BorderSize, XOfs, ClientHeight));
              FillRect(Rect(XOfs + DrawPict.Width, BorderSize, ClientWidth, ClientHeight));
            end;
            if YOfs > 0 then begin
              FillRect(Rect(BorderSize, BorderSize, ClientWidth, YOfs));
              FillRect(Rect(BorderSize, YOfs + DrawPict.Height, ClientWidth, ClientHeight));
            end;
          end
          else
            FillRect(ClientRect);
          StretchDraw(R, DrawPict.Graphic);
        end;
      finally
        DrawPict.Free;
      end;
    end
    else
    begin
      Font := Self.Font;
      if FDataLink.Field <> nil then
        S := FDataLink.Field.DisplayLabel
      else S := Name;
      S := '(' + S + ')';
      Size := TextExtent(S, 0);
      R := ClientRect;
      TextRect(R, (R.Right - R.Left - Size.cx) div 2 + BorderSize, (R.Bottom - R.Top - Size.cy) div 2 + BorderSize, S);
    end;
    Form := GetParentForm(Self);
    if (Form <> nil) and (Form.ActiveControl = Self) and
      not (csDesigning in ComponentState) and
      not (csPaintCopy in ControlState) then
    begin
      Brush.Color := clWindowFrame;
      Brush.Style := bsClear;
      Rectangle(ClientRect);
    end;
  end;
end;

procedure TCDBImage.TextChanged;
begin
  inherited;
  if not FPictureLoaded then Invalidate;
end;

procedure TCDBImage.PictureChanged(Sender: TObject);
begin
  if FPictureLoaded then FDataLink.Modified;
  FPictureLoaded := True;
  Invalidate;
end;

procedure TCDBImage.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TCDBImage.LoadPicture;
begin
  if not FPictureLoaded and (not Assigned(FDataLink.Field) or
    FDataLink.Field.IsBlob) then
{}  begin
{}    if (CompressSource=nil) or (not Assigned(FDataLink.Field)) then
        Picture.Assign(FDataLink.Field)
{}    else
{}       ReadCompressedPicture(CompressSource,Picture,TBlobField(FDataLink.Field));
{}  end;
end;

procedure TCDBImage.DataChange(Sender: TObject);
begin
  Picture.Graphic := nil;
  FPictureLoaded := False;
  if FAutoDisplay then LoadPicture;
end;

procedure TCDBImage.UpdateData(Sender: TObject);
begin
  if Picture.Graphic is TBitmap then
{}begin
{}    if CompressSource=nil then
         FDataLink.Field.Assign(Picture.Graphic)
{}    else
{}       WriteCompressedPicture(CompressSource,CompressionMethod,Picture,TBlobField(FDataLink.Field));
{}  end else
     FDataLink.Field.Clear;
end;

procedure TCDBImage.CopyToClipboard;
begin
end;

procedure TCDBImage.CutToClipboard;
begin
  if Picture.Graphic <> nil then
    if FDataLink.Edit then
    begin
      CopyToClipboard;
      Picture.Graphic := nil;
    end;
end;

procedure TCDBImage.PasteFromClipboard;
begin
end;

procedure TCDBImage.BoundsChanged;
begin
  inherited;
  Invalidate;
end;

procedure TCDBImage.DoEnter;
begin
  Invalidate; { Draw the focus marker }
  inherited;
end;

procedure TCDBImage.DoExit;
begin
  try
    if Assigned(DataSource) and Assigned(DataSource.DataSet) and
       (DataSource.DataSet.State in [dsInsert, dsEdit]) then
      FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;
  Invalidate; { Erase the focus marker }
  inherited;
end;

procedure TCDBImage.DblClick;
begin
  LoadPicture;
  inherited;
end;

function TCDBImage.GetClientRect: TRect;
begin
  Result := inherited GetClientRect;
  InflateRect(Result, - 2 * BorderSize, - 2 * BorderSize);
//  AdjustForClient(Result);
end;

procedure TCDBImage.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  case Key of
    Key_Insert:
      if ssShift in Shift then PasteFromClipBoard else
        if ssCtrl in Shift then CopyToClipBoard;
    Key_Delete:
      if ssShift in Shift then CutToClipBoard;
  end;
end;

procedure TCDBImage.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  case Key of
    ^X: CutToClipBoard;
    ^C: CopyToClipBoard;
    ^V: PasteFromClipBoard;
    #13: LoadPicture;
    #27: FDataLink.Reset;
  end;
end;

function TCDBImage.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TCDBImage.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

{} // TCompress:
procedure Register;
begin
  RegisterComponents('Compress', [TCDBMemo,TCDBImage]);
end;

end.

